import { IDictionary, IEventParam, Metadata, Singleton } from '@zeedhi/core';
import {
	ITreeClickEvent,
	ITreeNodeModel,
	Modal,
	ModalService, Select,
	Tree,
} from '@zeedhi/common';
import { PmLibDialogService, PmLibFormSave, PmLibHttp, PmLibAlertService } from '@zeedhi/pm-lib';
import { saveAs } from 'file-saver';
import Messages from '@/services/Messages';
import { ZdTextInput } from '@zeedhi/vuetify';
import { NodeType } from '@/services/AclLabels/NodeType';
import { NodeTypeService } from '@/services/AclLabels/NodeTypeService';
import Store from '@/store';
import { AclLabelsService } from '@/services/AclLabels/AclLabelsService';

@Singleton
export class AclLabelsController {
	/**
	 * Properties aclLabelsTree
	 */
	get originalNodes() {
		return Store.getters['AclLabels/getOriginalNodes'];
	}

	set originalNodes(value: any) {
		Store.commit('AclLabels/setOriginalNodes', value);
	}

	public hasNoSelectedNodes: boolean = true;

	public hasMultipleSelectedNodes: boolean = false;

	get hasChanges(): boolean {
		return Store.getters['AclLabels/getHasChanges'];
	}

	set hasChanges(value: boolean) {
		Store.commit('AclLabels/setHasChanges', value);
	}

	get version(): number {
		return Store.getters['AclLabels/getVersion'];
	}

	set version(value: number) {
		Store.commit('AclLabels/setVersion', value);
	}

	get aclLabelsTreeComponent(): Tree {
		return Store.getters['AclLabels/getAclLabelsTreeComponent'];
	}

	get nodeTypeValue(): string {
		return Store.getters['AclLabels/getNodeTypeValue'];
	}

	set nodeTypeValue(value: string) {
		Store.commit('AclLabels/setNodeTypeValue', value);
	}

	get addLabelModal(): Modal {
		return Store.getters['AclLabels/getAddLabelModal'];
	}

	get importMenuModal(): Modal {
		return Store.getters['AclLabels/getImportMenuModal'];
	}

	get duplicateLabelsModal(): Modal {
		return Store.getters['AclLabels/getDuplicateLabelsModal'];
	}

	public getNodeTitle(node: ITreeNodeModel<any>) {
		return node.title;
	}

	public getNodeLabel(node: ITreeNodeModel<any>) {
		return node.data.label;
	}

	private async getAclLabels() {
		const projectId = Store.getters['AclLabels/getCurrentProject'].PROJECT_ID;
		await Store.dispatch('AclLabels/loadNodes', projectId);
	}

	public getNodeTypeLabel(node: ITreeNodeModel<any>) {
		return NodeTypeService.getNodeTypeLabel(node.data.type);
	}

	public getNodeTypeColor(node: ITreeNodeModel<any>) {
		return NodeTypeService.getNodeTypeColor(node.data.type);
	}

	/**
	 * Events methods aclLabelsTree
	 */

	public async onCreatedAclLabelsTree() {
		await Store.dispatch('AclLabels/loadAddLabelModal');
		this.editLabelModal = await ModalService.createFromJson(this.editLabelModalPath, true);
		this.openFileModal = await ModalService.createFromJson(this.openFileModalPath, true);
		await Store.dispatch('AclLabels/loadImportMenuModal');
		await Store.dispatch('AclLabels/loadDuplicateLabelsModal');
	}

	public onBeforeDestroyAclLabelsTree() {
		this.addLabelModal?.destroy();
		this.editLabelModal?.destroy();
		this.openFileModal?.destroy();
		this.importMenuModal?.destroy();
		this.duplicateLabelsModal?.destroy();
	}

	public onNodeSelectAclLabelsTree({ component }: IEventParam<Tree>) {
		this.hasNoSelectedNodes = component.selectedNodes.length === 0 || this.hasNodeMenuType(component.selectedNodes);
		this.hasMultipleSelectedNodes = this.hasMultipleSelected(component.selectedNodes);
	}

	private hasMultipleSelected(selectedNodes: any) {
		if (selectedNodes.length > 1) {
			return true;
		}

		if (selectedNodes.length === 1) {
			const node = selectedNodes[0];
			return this.nodeIsRuleType(node);
		}

		return false;
	}

	private nodeIsRuleType(node: ITreeNodeModel<any>) {
		return node.data?.type === 'RULE';
	}

	private hasNodeMenuType(nodes: Array<ITreeNodeModel<any>>): boolean {
		return nodes.some((node) => node.data.type === NodeType.menu);
	}

	public async onNodeDblClickAclLabelsTree({ node }: ITreeClickEvent) {
		await this.setNodeTypeValueByUpdate(node);
	}

	private async setNodeTypeValueByUpdate(node: ITreeNodeModel<any>) {
		if (node.data.type === NodeType.grouped) {
			this.nodeTypeValue = NodeType.grouped;
			await this.loadEditLabelModal(node);
		} else if (node.data.type === NodeType.rule) {
			this.nodeTypeValue = NodeType.rule;
			await this.loadEditLabelModal(node);
		}
	}

	private async loadEditLabelModal(node: any) {
		await this.editLabelModal?.show();
		const editForm = this.getEditLabelModalForm();
		this.editNode = node;
		this.editNodeBackup = node;
		editForm.value.name = node.title;
		editForm.value.label = node.data?.label;
	}

	/**
	 * Actions methods aclLabelsTree
	 */

	public clickAclLabelsTreeButtonAdd() {
		const nodes = this.aclLabelsTreeComponent.selectedNodes;
		if (nodes.length && nodes.length === 1) {
			const node = nodes[0];
			if (node.data?.type === NodeType.grouped || node.data?.type === NodeType.menu) {
				this.nodeTypeValue = NodeType.rule;
				this.addLabelModal?.show();
			}
		} else if (!nodes.length) {
			this.nodeTypeValue = NodeType.grouped;
			this.addLabelModal?.show();
		}
	}

	public async clickAclLabelsTreeButtonDelete() {
		const response = await PmLibDialogService.confirmDialog('Deseja excluir o(s) Label(s) selecionado(s)');
		if (response) {
			this.aclLabelsTreeComponent.removeSelected();
			this.hasMultipleSelectedNodes = false;
			await this.clickAclLabelsTreeButtonSave();
		}
	}

	public async clickAclLabelsTreeButtonReload() {
		await this.getAclLabels();
		this.hasChanges = false;
		this.aclLabelsTreeComponent.clearSelection();
		this.hasNoSelectedNodes = true;
		this.hasMultipleSelectedNodes = false;
	}

	public clickAclLabelsTreeButtonCancel() {
		if (this.originalNodes) {
			this.aclLabelsTreeComponent.nodes = [...this.originalNodes];
		} else {
			this.aclLabelsTreeComponent.nodes = [];
		}
		this.hasChanges = false;
		this.aclLabelsTreeComponent.clearSelection();
	}

	public async clickAclLabelsTreeButtonSave() {
		const value = this.aclLabelsTreeComponent.nodes;
		this.hasChanges = false;
		this.originalNodes = [...value];
		this.aclLabelsTreeComponent.clearSelection();
		const labelsFileString = JSON.stringify(this.originalNodes);
		const projectId = Store.getters['AclLabels/getCurrentProject'].PROJECT_ID;
		const data = { labelsFile: labelsFileString, projectId };
		await PmLibHttp.$post('/projects/aclLabels', data);
		PmLibAlertService.showSuccess(Messages.SUCCESS_SAVE_DEAFULT);
	}

	public clickAclLabelsTreeButtonOpenFile() {
		this.openFileModal?.show();
	}

	public clickAclLabelsTreeButtonDownload() {
		let nameArchive;
		const value = this.aclLabelsTreeComponent.nodes;
		try {
			const openFileForm = Metadata.getInstance<PmLibFormSave>('openFileModalForm');
			nameArchive = openFileForm.value.fileJson[0].name;
		} catch (error) {
			nameArchive = 'aclLabels.json';
		}
		const blob = new Blob([JSON.stringify(value)], { type: 'text/plain;charset=utf-8' });
		saveAs(blob, nameArchive);
		this.hasChanges = false;
		this.originalNodes = [...value];
		this.version += 1;
	}

	public async clickAclLabelsTreeButtonImportMenu() {
		this.importMenuModal?.show();
	}

	/**
	 * Edit label Modal Controller
	 */

	public editLabelModal!: Modal;

	private editLabelModalPath = 'aclLabels/components/detail/tabs/labels/modais/editLabel';

	private editNode?: any;

	private editNodeBackup: any = {};

	public onMountedEditLabelModalForm() {
		const editLabelForm = Metadata.getInstance<PmLibFormSave>('editLabelModalForm');
		const typeField = editLabelForm.getChildInstance<Select>('type');
		const labelField = editLabelForm.getChildInstance<ZdTextInput>('label');
		labelField.readonly = this.nodeTypeValue === NodeType.grouped;
		typeField.setValue(this.nodeTypeValue);
	}

	public closeEditLabelModalForm() {
		this.editLabelModal?.hide();
	}

	public getEditLabelModalForm(): PmLibFormSave {
		return Metadata.getInstance<PmLibFormSave>('editLabelModalForm');
	}

	public saveEditLabelModalForm() {
		const formEdit = this.getEditLabelModalForm();
		if (formEdit.validate() && this.validateDuplicateLabels(formEdit)) {
			const newItem = this.getNewItem(this.getEditLabelModalForm());
			this.aclLabelsTreeComponent.updateNodeData(this.editNode, newItem);
			this.hasChanges = true;
			this.editLabelModal?.hide();
			this.editNodeBackup = {};
		}
	}

	private validateDuplicateLabels(editLabelForm: PmLibFormSave): boolean {
		if (editLabelForm.value.type === NodeType.rule && this.editNodeBackup.data.label !== editLabelForm.value.label) {
			const { label } = editLabelForm.value;
			const allLabels = AclLabelsService.getAllLabelsInNodes(this.aclLabelsTreeComponent.nodes);
			if (allLabels.includes(label)) {
				PmLibDialogService.showError(Messages.MSG_DUPLICATE_LABEL_SAVE);
			}
			return !allLabels.includes(label);
		}

		return true;
	}

	/**
	 * Open File Modal Controller
	 */

	public openFileModal!: Modal;

	private openFileModalPath = 'aclLabels/components/detail/tabs/labels/modais/openFile';

	private errorMsgFileOpen = 'Permitidos apenas arquivos .json válidos';

	public closeOpenFileModalForm() {
		this.openFileModal?.hide();
	}

	public changeInputFileJson() {
		try {
			const openFileForm = Metadata.getInstance<PmLibFormSave>('openFileModalForm');
			const readFile = openFileForm.value.fileJson;
			const reader = new FileReader();

			reader.onload = (evt: any) => {
				try {
					const nodesString = evt.target.result.replaceAll('"isSelected":true', '"isSelected":false');
					this.aclLabelsTreeComponent.nodes = JSON.parse(nodesString);
					this.version = this.getLastVersion();
					this.hasNoSelectedNodes = true;
				} catch (e) {
					this.hasChanges = false;
					PmLibDialogService.showError(this.errorMsgFileOpen);
				}
			};

			if (openFileForm.validate()) {
				reader.readAsText(readFile);
				this.openFileModal?.hide();
				this.hasChanges = true;
			}
		} catch (error) {
			this.hasChanges = false;
			PmLibDialogService.showError(this.errorMsgFileOpen);
		}
	}

	/**
	 * Aux
	 */

	public getNewItem(form: PmLibFormSave): any {
		const newItem: any = {
			title: form.value.name,
			data: {
				version: this.version + 1,
				type: form.value.type,
			},
		};

		if (form.value.label && form.value.type !== NodeType.grouped) {
			newItem.data.label = form.value.label;
		}

		return newItem;
	}

	private getLastVersion() {
		let version = 0;
		this.aclLabelsTreeComponent.nodes.forEach((node) => {
			if (node.data && node.data.version > version) {
				// eslint-disable-next-line prefer-destructuring
				version = node.data.version;
			}
			version = this.getLastChildrenVersion(node, version);
		});

		return version;
	}

	private getLastChildrenVersion(node: ITreeNodeModel<IDictionary<any>>, version: number) {
		let childrenVersion = version;
		if (node.children) {
			node.children.forEach((child) => {
				if (child.data && child.data.version > childrenVersion) {
					childrenVersion = child.data.version;
				}

				childrenVersion = this.getLastChildrenVersion(child, childrenVersion);
			});
		}
		return childrenVersion;
	}
}
