import { Metadata, Singleton } from '@zeedhi/core';
import { IMenuItem, ITreeNodeModel, LoadingService, Modal, Tree } from '@zeedhi/common';
import Store from '@/store';
import { PmLibDialogService, PmLibFormSave, PmLibUtil } from '@zeedhi/pm-lib/index';
import axios from 'axios';
import { NodeType } from '@/services/AclLabels/NodeType';
import Messages from '@/services/Messages';

@Singleton
export class AclLabelsImportMenuModalController {
	private newNodes: Array<ITreeNodeModel<any>> = [];

	private newNodesLabel: Array<string> = [];

	private newNodesLabelDuplicated: Array<string> = [];

	get importMenuModal(): Modal {
		return Store.getters['AclLabels/getImportMenuModal'];
	}

	get originalNodes(): Array<ITreeNodeModel<any>> {
		return Store.getters['AclLabels/getOriginalNodes'];
	}

	set originalNodes(value: Array<ITreeNodeModel<any>>) {
		Store.commit('AclLabels/setOriginalNodes', value);
	}

	get aclLabelsTreeComponent(): Tree {
		return Store.getters['AclLabels/getAclLabelsTreeComponent'];
	}

	get hasChanges(): boolean {
		return Store.getters['AclLabels/getHasChanges'];
	}

	set hasChanges(value: boolean) {
		Store.commit('AclLabels/setHasChanges', value);
	}

	get version(): number {
		return Store.getters['AclLabels/getVersion'];
	}

	set version(value: number) {
		Store.commit('AclLabels/setVersion', value);
	}

	get duplicateLabelsModal(): Modal {
		return Store.getters['AclLabels/getDuplicateLabelsModal'];
	}

	public closeImportMenuModalForm() {
		this.importMenuModal?.hide();
	}

	public async saveImportMenuModalForm() {
		try {
			LoadingService.show();
			this.originalNodes = PmLibUtil.clone(this.aclLabelsTreeComponent.nodes);
			const importMenuForm = Metadata.getInstance<PmLibFormSave>('importMenuModalForm');
			const menuURL = importMenuForm.value.url;
			const { data } = await axios.post(menuURL);
			this.import(JSON.stringify(data));
			this.aclLabelsTreeComponent.clearSelection();
			this.hasChanges = true;
			this.importMenuModal?.hide();
			LoadingService.hide();
			this.resetStateFile();
		} catch (error) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(error);
		}
	}

	private import(text: string) {
		this.aclLabelsTreeComponent.nodes = [];
		this.newNodes = this.getNodesFromMenu(text);

		this.importOriginalRulesToNewNodes();

		if (this.newNodesLabelDuplicated.length) {
			this.checkDuplicateLabels();
		} else if (!this.originalNodes.length) {
			this.aclLabelsTreeComponent.importNodes(this.newNodes);
			this.setNodeTypeGroupedDefault();
		} else {
			this.copyNodesTypeGrouped();
			this.aclLabelsTreeComponent.importNodes(this.newNodes);
		}
	}

	private getNodesFromMenu(json: string) {
		const importObject = JSON.parse(json);
		const items = importObject.dataset?.menu || importObject.menu || importObject;
		return this.getNodes(items);
	}

	private getNodes(items: IMenuItem[]) {
		return items.map((item) => {
			const node: any = {
				title: item.label,
				data: {
					version: this.version + 1,
					label: this.createLabel(item.label),
					type: 'MENU',
				},
			};

			if (item.items) {
				node.children = this.getNodes(item.items);
			} else if (item.menus?.length) {
				node.children = this.getNodes(item.menus);
			} else {
				node.children = [];
			}

			return node;
		});
	}

	private setNodeTypeGroupedDefault() {
		const newItem: any = {
			title: 'Gerais',
			data: {
				version: 1,
				type: 'GROUPED',
			},
		};

		this.aclLabelsTreeComponent.insertNode(newItem);
	}

	private copyNodesTypeGrouped() {
		this.originalNodes.forEach((node) => {
			if (node.data.type === NodeType.grouped) {
				this.newNodes.push(node);
			}
		});
	}

	private createLabel(label: string) {
		return this.removeAccent(label).toUpperCase().replaceAll(' ', '_');
	}

	private removeAccent(text: string) {
		return text.normalize('NFD').replace(/[\u0300-\u036f]/g, '');
	}

	private importOriginalRulesToNewNodes() {
		this.iterateNewNodes(this.newNodes);
	}

	private iterateNewNodes(newNodes: Array<ITreeNodeModel<any>> | Array<any>) {
		newNodes.forEach((node) => {
			this.addDuplicateLabels(node);
			this.iterateNewOriginalNodes(node, this.originalNodes);
			this.iterateNewNodes(node.children);
		});
	}

	private iterateNewOriginalNodes(newNode: ITreeNodeModel<any>, originalNodes: Array<ITreeNodeModel<any>> | Array<any>) {
		originalNodes.some((originalNode) => {
			if (newNode.data.type === NodeType.menu && newNode.data.label === originalNode.data.label) {
				this.addRuleTypeNode(newNode, originalNode);
			} else {
				this.iterateNewOriginalNodes(newNode, originalNode.children);
			}
			return newNode.data.label === originalNode.data.label;
		});
	}

	private addRuleTypeNode(newNode: ITreeNodeModel<any>, originalNode: ITreeNodeModel<any>) {
		originalNode.children?.forEach((child) => {
			if (child.data.type === NodeType.rule) {
				newNode.children?.push(child);
			}
		});
	}

	private addDuplicateLabels(node: ITreeNodeModel<any>) {
		if (this.newNodesLabel.includes(node.data.label)) {
			this.newNodesLabelDuplicated.push(node.data.label);
		}
		this.newNodesLabel.push(node.data.label);
	}

	private checkDuplicateLabels() {
		PmLibDialogService.showError(Messages.MSG_DUPLICATE_LABEL_IMPORT_MENU);
		this.duplicateLabelsModal.show();
	}

	private resetStateFile() {
		this.newNodes = [];
		this.newNodesLabel = [];
		this.newNodesLabelDuplicated = [];
	}
}
